; This is some code to test memory paging and 128K file saving
; Has a vaguely useful proportional print routine

AppFilename             equ "Mode3"                     ; What we're called (for file generation)

                        zeusemulate "128K"              ; Set a model with 128K of RAM

; Comment this out if you just want it to sit there doing nothing

bRinceAndRepeat         equ true                        ; If enabled, we repeat the display

; Print control codes

cDel                    equ $08                         ;
cCret                   equ $0D                         ;
cCls                    equ $0C                         ;
cClr                    equ $1B                         ;
cHome                   equ $0E                         ;

; Print colour select characters

cBlack                  equ $00                         ;
cBlue                   equ $01                         ;
cRed                    equ $02                         ;
cMagenta                equ $03                         ;
cGreen                  equ $04                         ;
cCyan                   equ $05                         ;
cYellow                 equ $06                         ;
cWhite                  equ $07                         ;

; For this example we want to build code that will run at location $C000, but we're going to
; have Zeus do that for all 8 memory pages.
;
; So the first thing this code illustrates is how to use pages in Zeus. Basically Zeus can address
; more than 64K of memory, and Zeus uses this to map the pages to addresses higher than 64K. It is
; important to understand that this means some of the pages can be "seen" at two different places
; in Zeus - for example a 128K Spectrum maps page 5 at $4000, and so does Zeus. But Zeus *also*
; maps page 5 at $24000... so the same byte is visible at $4000 and at $24000. As far as Zeus is
; concerned it doesn't matter which you use.
;
; So, how do you use Zeus to write code for paged memory?
;
; Well, to put data in page 0 you could simply do this:
;
;               org $10000 ; <- note that is NOT $1000, it's 65536d
;               db "Hello, this text is being put in page 0"
;
; But if you are trying to build code that's going to run in a page it's not enough to just tell
; Zeus where to put it, you have to know what address it's going to be mapped at when it's run and
; tell Zeus that (so the addresses Zeus uses in the code are right) but also tell Zeus that Zeus
; should actually write the bytes to a different area of memory. This is known as "DISPLACING" the
; code and you can do this a number of different ways in Zeus.
;
; First there is the DISP command. DISP is simple, it just stores the displacement you
; put after DISP and Zeus adds that displacement to the addresses it uses when it stores code.
;
; So, for example, if you did this:
;
;               org $4000       ; Build code to run at $4000
;               disp $1000      ; Put put it in memory at $5000 (which is $4000 + $1000)
;               db "This string is put in memory starting at $5000",0
;
; Then there is the DISPTO operator. You give this the address you want code displaced to and it
; works out what displacement needs to be applied to put the next byte written at that address.;
;
; So, for example, if you did this:
;
;               org $4000       ; Build code to run at $4000
;               dispto $5000    ; Put put it in memory at $5000
;               db "This string is put in memory starting at $5000",0
;
; Finally, you can set a displacement directly when using the ORG statement as an extra parameter
;
; So, for example, you can do this:
;
;               org $4000,$1000 ; Build code to run at $4000, but put it in memory at $5000
;               db "This string is put in memory starting at $5000",0
;
; So, in practice let's say we want to page memory using the top 16K block (since the Spectrum memory
; paging is so limited that this is the best option) and this means that we want the code to be
; running at $C000..$FFFF, so we need to use "ORG $C000" to tell Zeus to build the code to run
; at addresses starting from $C000... but if we actually want it put in code page 0, which starts at
; $10000, we need to have a "DISP $4000" which will add $C000+$4000 to get $10000
;
; In practice it's better not to have to work out what actual DISP value we need but to let Zeus do
; that calculation for us. So what we do is "ORG $C000" followed by "DISPTO [requiredpageaddress]"
; So for example "DISPTO $10000" for page 0, "DISPTO $14000" for page 1, and so on...
;
; And to save you having to remember these page addresses, Zeus provides a function to do this, so
; you can do it as simply as this:
;
;               org $C000       ; Build code to run at $C000
;               dispto zeuspage(0) ; But put it in memory at $10000
;
; Something to remember is that you need to set the displacement to zero after you have finished
; with the section of displaced code or data. "DISP 0" does this.
;
; Now, another issue with paging memory is that only one page is "visible" to the Z80 at any given
; time. So you have to be careful when writing code not to have any of it refer to things that are
; not visible at the time it's running... if you have page 0 mapped at $C000, you cannot see anything
; in any other page at $C000.
;
; One way Zeus can help you with this is to use the proc/endp structure to make all the labels
; in a page local to a single procedure. That way code in one page/procedure cannot simply
; refer to the labels in another page using their name, it has to use the procedure name prefix.
; Nor can code outside the page simply refer to routines in it without the prefix. But code in
; any given page can refer to local lables as normal... and code in any page can re-use the same
; names for different addresses without clashing.
;
; This example code shows all of these techniques:

; This is the code for RAM page 0 and we're going to put it inside a procedure with the name
; "CodePage_0", because that will remind us which page it's in whenever we use it.

CodePage_0              proc                            ; CodePage_0 is the name of this procedure

                        ; All the labels from this point on are local to procedure "CodePage_0"
                        ; In modern parlance we'd probably call procedures "namespaces"

                        ; We build code that will run at $C000, but put it at $10000
                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(0)              ; But put it in memory at $10000

String                  db "This string is in page 0",0 ; See below for how to use this label
PageLen                 equ *-String                    ; The length
                        pend                            ; End of procedure, back to global scope.

; This is the code for RAM page 1

CodePage_1              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(1)              ; But put it in memory at $14000

                        ; Let's pretend we have some code here

                        nop                             ; Some bytes
                        nop                             ; Some bytes
                        nop                             ; Some bytes

; Now! Pay attention. This string uses the same name as the string in Page 0, but it is NOT at the
; same address. That one started at $C000, this one starts at $C003
                        noflow                          ;
String                  db "This string is in page 1",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 2

CodePage_2              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(2)              ; But put it in memory at page 2's address

String                  db "This string is in page 2",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 3

CodePage_3              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(3)              ; But put it in memory at page 3's address

String                  db "This string is in page 3",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 4

CodePage_4              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(4)              ; But put it in memory at page 4's address

String                  db "This string is in page 4",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 5

CodePage_5              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(5)              ; But put it in memory at page 5's address

                        ds $2000                        ; We need to avoid the screen memory in Page 5
String                  db "This string is in page 5",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 6

CodePage_6              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(6)              ; But put it in memory at page 6's address

                        ds $2000,$66                    ;
String                  db "This string is in page 6",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; This is the code for RAM page 7

CodePage_7              proc                            ;

                        org $C000                       ; Build the code to run at $C000
                        dispto zeuspage(7)              ; But put it in memory at page 7's address

String                  db "This string is in page 7",0 ;
PageLen                 equ *-String                    ; The length

                        pend                            ;

; We put the code out of the way at $9000, and we clear the displacement

                        org $9000,0                     ; Start of application (And clear the displacement)

AppEntry                di                              ; Make sure we're not interrupted
                        ld sp,$C000                     ; Set the stack to the top of memory

                        ld a,cCls                       ; Clear the screen
                        call Print                      ;

Loopy                   ld a,cBlue                      ; Show them in blue
                        call Print                      ;

                        ; Page 0

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#0                         ; Start with page 0
                        out (c),a                       ;

                        ; Now! Pay attention to how Zeus refers to local labels from outside the
                        ; procedure containing them, using "procedurename.localname"
                        ; So this next line gets the address of the String in page 0, which is $C000

                        ld hl,CodePage_0.String         ; Point at the string in page 0
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 1

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#1                         ; Page 1
                        out (c),a                       ;

                        ; This next line gets the address of the String in page 1, which is $C003
                        ld hl,CodePage_1.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 2

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#2                         ; Page 2
                        out (c),a                       ;

                        ld hl,CodePage_2.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 3

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#3                         ; Page 3
                        out (c),a                       ;

                        ld hl,CodePage_3.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 4

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#4                         ; Page 4
                        out (c),a                       ;

                        ld hl,CodePage_4.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 5

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#5                         ; Page 5
                        out (c),a                       ;

                        ld hl,CodePage_5.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 6

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#6                         ; Page 6
                        out (c),a                       ;

                        ld hl,CodePage_6.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; Page 7

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#7                         ; Page 7
                        out (c),a                       ;

                        ld hl,CodePage_7.String         ; Point at the string in this page
                        call PrintStr                   ; Show the value
                        ld a,cCret                      ; CR
                        call Print                      ;

                        ; We're done...

        if enabled bRinceAndRepeat                      ;

                        ; Make sure we sit in page 0 so emulators save snapshots in a known page state

                        ld bc,$7FFD                     ; The memory paging port
                        ld a,#0                         ; Start with page 0
                        out (c),a                       ;

                        ; Wait a while

                        ld bc,0                         ; Look, I just wanted the sodding emulator to wait a bit

Pause                   push ix                         ; Delay
                        pop ix                          ;

                        cpi                             ; Loop
                        jp v Pause                      ;

                        jp Loopy                        ; Display them again
        endif                                           ;

                        halt                            ; Stop all the excitement

; Print the zero-terminated string pointed to by HL

PrintStr                proc                            ;

                        push hl,af                      ;

Loop                    ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,Exit                       ;

                        call Print                      ;
                        jr Loop                         ;

Exit                    pop af,hl                       ;
                        retp                            ; (RET)

; Print the string following the call in memory

PrintStrFollow          proc                            ;

                        ex (sp),hl                      ;
                        push af                         ;

Loop                    ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,Exit                       ;

                        call Print                      ;

                        jr Loop                         ;

Exit                    pop af                          ;
                        ex (sp),hl                      ;

                        retp                            ; (RET)


; Print the contents of HL in hex

PrintHex_HL             ld a,h                          ;
                        call PrintHex_A                 ;

                        ld a,l                          ;

                        ; Print the contents of A in hex

PrintHex_A              call PH2                        ;

PH2                     rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;

                        push af                         ;

                        and $0F                         ;

                        add a,"0"                       ;
                        cp "9"+1                        ;
                        jr c,PH3                        ;

                        add a,"A"-("9"+1)               ;

PH3                     call Print                      ;

                        pop af                          ;
                        ret                             ;

; Print a space

Print_Sp                push af                         ; Save

                        ld a,' '                        ; Print a space
                        call Print                      ;

                        pop af                          ; Done
                        ret                             ;

; A print character routine. Proportionally spaced characters

Print                   push ix,hl,de,bc,af             ; Save them

                        cp cWhite+1                     ;
                        jr c,PrintColour                ;

                        cp cCls                         ;
                        jr z,PrintCls                   ;

                        cp cCret                        ;
                        jr z,PrintCret                  ;

                        cp cHome                        ;
                        jr z,PrintHome                  ;

                        ld h,PrintWidths/256            ;
                        ld l,a                          ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        call c,PrintCret                ;

                        call PrintOutChar               ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        ld (pCurX),a                    ;

PrintExit               pop af,bc,de,hl,ix              ;
                        ret                             ;

; Set a colour

PrintColour             cp 4                            ; Above green don't make them bright
                        jr nc,PrintCol1                 ;

                        or $40                          ; Bright

PrintCol1               ld (CurCol),a                   ;
                        jr PrintExit                    ;

; Carriage return

PrintCret               call PrintDoCret                ;
                        jr PrintExit                    ;

; Clear screen - this clears it slowly with a fade effect

PrintCls                ld e,$00                        ;
                        ld hl,$5800                     ;
                        ld bc,$02FF                     ;

pC1                     ld a,(hl)                       ;
                        and $07                         ;
                        jr z,pC2                        ;

                        set 0,e                         ;
                        dec (hl)                        ;
pC2                     cpi                             ;
                        jp pe,pC1                       ;

                        call Delay                      ;

                        bit 0,e                         ;
                        jr nz,PrintCls                  ;

                        ld hl,$4000                     ;
                        ld de,$4001                     ;
                        ld bc,$1800                     ;

                        xor a                           ;
                        ld (hl),a                       ;

                        ldir                            ;

                        ld a,(CurCol)                   ;
                        ld (hl),a                       ;

                        ld bc,$02FF                     ;

                        ldir                            ;

                        xor a                           ;
                        out ($FE),a                     ;

PrintHome               xor a                           ;
                        ld (pCurX),a                    ;
                        ld (ppCurY),a                   ;

                        ld a,cWhite                     ;
                        ld (CurCol),a                   ;

                        ld hl,$4000                     ;
                        ld (pCurY),hl                   ;

                        jr PrintExit                    ;

; A suitable delay for the fade

Delay                   push bc                         ;
                        ld bc,1000                      ;
Del1                    cpi                             ;
                        dec hl                          ;
                        jp pe,Del1                      ;
                        pop bc                          ;
                        ret                             ;

; Do the carriage return

PrintDoCret             ld a,(ppCurY)                   ;
                        inc a                           ;

PrintSetUpY             cp 24                           ; Gone off the bottom?
                        jr c PSUY_Ok                    ; No, it's Okay

                        ; Scroll the screen up one line

                        call ScrollUp                   ;

                        ld a,23                         ; Bottom line

PSUY_Ok                 ld (ppCurY),a                   ;

                        and $1F                         ;

                        push hl                         ;

                        ld l,a                          ;
                        ld h,$00                        ;

                        add hl,hl                       ;

                        ld de,YTable                    ;
                        add hl,de                       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        ld (pCurY),hl                   ;

                        xor a                           ;
                        ld (pCurX),a                    ;

                        pop hl                          ;
                        ret                             ;

; Output the character

PrintOutChar            bit 7,l                         ; characters $00..$7F only
                        ret nz                          ;

                        push hl                         ;

                        ld h,0                          ;

                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;

                        ld de,PrintChars-256            ;
                        add hl,de                       ;

                        push hl                         ;
                        pop ix                          ;

                        ld a,(pCurX)                    ;

                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $1F                         ;

                        ld de,(pCurY)                   ;

                        or e                            ;
                        ld e,a                          ;

                        push de                         ;

                        ld a,d                          ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $03                         ;
                        or $58                          ;
                        ld d,a                          ;

                        ld a,(CurCol)                   ;
                        ld (de),a                       ;
                        inc de                          ;
                        ld (de),a                       ;

                        pop de                          ;

                        ld a,(pCurX)                    ;
                        and $07                         ;
                        ld c,a                          ;

                        ld b,$08                        ;

OutCharL                ld h,(ix)                       ;
                        ld l,$00                        ;

                        ld a,c                          ;
                        or a                            ;
                        jr z,OCL1                       ;

OCL2                    srl h                           ;
                        rr l                            ;
                        dec a                           ;
                        jr nz,OCL2                      ;

OCL1                    ld a,(de)                       ;
                        xor h                           ;
                        ld (de),a                       ;

                        inc e                           ;

                        ld a,(de)                       ;
                        xor l                           ;
                        ld (de),a                       ;

                        dec e                           ;

                        inc ix                          ;
                        inc d                           ;

                        djnz OutCharL                   ;

                        pop hl                          ;
                        ret                             ;

ScrollUp                push ix,hl,de,bc                ; Save

                        ld ix,YTable                    ;
SU_Lp                   ld hl,(ix+2)                    ;

                        ld a,l                          ;
                        or h                            ;
                        jr z SU_Moved                   ;

                        ld de,(ix)                      ;

                        inc ix                          ;
                        inc ix                          ;

                        push hl,de                      ;

                        ld a,d                          ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $03                         ;
                        or $58                          ;
                        ld d,a                          ;

                        ld a,h                          ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $03                         ;
                        or $58                          ;
                        ld h,a                          ;

                        ld bc,32                        ;
                        ldir                            ;
                        pop de,hl                       ;



SU_Lp2                  push hl,de                      ;
                        ld bc,32                        ;
                        ldir                            ;
                        pop de,hl                       ;

                        inc h                           ;
                        inc d                           ;

                        ld a,h                          ;
                        and %00000111                   ;
                        jr nz SU_Lp2                    ;

                        jr SU_Lp                        ; Loop

                        ; Clear the bottom line

SU_Moved                ld hl,$50E0                     ; Clear the bottom line
                        call ClearLine                  ;

                        pop bc,de,hl,ix                 ;
                        ret                             ;

                        ; Clear a line

ClearLine               push hl                         ;

                        ld d,h                          ;
                        ld e,l                          ;
                        inc de                          ;

                        ld (hl),0                       ;
                        ld bc,31                        ;
                        ldir                            ;

                        pop hl                          ;

                        inc h                           ;

                        ld a,h                          ;
                        and %00000111                   ;
                        jr nz ClearLine                 ;

                        ret                             ; Done

; Build a table for the screen addresses for the start of each character row

YTable                  for screenline=0 to 191 step 8  ;
                        dw zxpixeladdr(0,screenline)    ;
                        next                            ;

                        dw 0,0,0,0,0,0,0,0              ; Add some dummy lines so running off the end isn't fatal

; Character set graphics. These will be used in a proportionally spaced print routine so we'll left-align them
; and provide the widths below

                        align 256                       ; Put these on page boundaries

PrintChars              dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg #####---                     ;
                        dg -#-#----                     ;
                        dg #####---                     ;
                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -####---                     ;
                        dg #-#-----                     ;
                        dg -###----                     ;
                        dg --#-#---                     ;
                        dg ####----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg ##--#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #--##---                     ;
                        dg ---##---                     ;
                        dg --------                     ;

                        dg -#------                     ;
                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg -#------                     ;
                        dg #-#-#---                     ;
                        dg #--#----                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg #-#-#---                     ;
                        dg -###----                     ;
                        dg --#-----                     ;
                        dg -###----                     ;
                        dg #-#-#---                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg #-#-#---                     ;
                        dg ##--#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -##-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg ----#---                     ;
                        dg --##----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --##----                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ---#----                     ;
                        dg --##----                     ;
                        dg -#-#----                     ;
                        dg #--#----                     ;
                        dg #####---                     ;
                        dg ---#----                     ;
                        dg ---#----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --###---                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #-###---                     ;
                        dg #-##----                     ;
                        dg #-------                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ###-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #--#----                     ;
                        dg #-#-----                     ;
                        dg ##------                     ;
                        dg #-#-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg ##-##---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ##--#---                     ;
                        dg #-#-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #--#----                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-#-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg ##-##---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg ----#---                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -###----                     ;
                        dg #-#-#---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -##-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-##----                     ;
                        dg ##--#---                     ;
                        dg #---#---                     ;
                        dg ##--#---                     ;
                        dg #-##----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg -##-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg ----#---                     ;
                        dg -###----                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ##------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #--#----                     ;
                        dg #-#-----                     ;
                        dg ###-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ##-#----                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #-##----                     ;
                        dg ##------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg ----#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ---##---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg -#-#----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #--#----                     ;
                        dg #--#----                     ;
                        dg #--#----                     ;
                        dg ####----                     ;
                        dg ---#----                     ;
                        dg ###-----                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --##----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --##----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ##------                     ;
                        dg --------                     ;

                        dg -#-#----                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ----#---                     ;
                        dg -###----                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

; Character widths

PrintWidths             db 0,0,0,0,0,0,0,0              ;
                        db 0,0,0,0,0,0,0,0              ;
                        db 0,0,0,0,0,0,0,0              ;
                        db 0,0,0,0,0,0,0,0              ;
                        db 6,2,4,6,6,6,6,2              ;
                        db 4,4,6,6,3,6,2,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,2,3,5,6,5,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,4,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,4,6              ;
                        db 6,2,4,6,4,6,6,6              ;
                        db 6,6,5,6,6,6,6,6              ;
                        db 6,5,6,5,2,5,6,6              ;
                        db 3,6,12,18,24,30,36,42        ;
                        db 48,54,60,66,72,6,6,6         ;
                        db 1,2,3,4,5,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;
                        db 6,6,6,6,6,6,6,6              ;

; Variables

pCurX                   db 0                            ; A logical X position
ppCurY                  db 0                            ; A logical Y position
pCurY                   dw 0                            ; The actual address
CurCol                  db 0                            ; The cursor colour

; Stop planting code after this. (When generating a tape file we save bytes below here)
AppLast                 equ *-1                         ; The last used byte's address

; We need a loading screen

                        import_bin "Mode3.scr",$4000    ; A simple loading screen

; Generate some useful debugging commands

                        profile AppEntry,AppLast-AppEntry+1 ; Enable profiling for all the code

; Setup the emulation registers, so Zeus can emulate this code correctly

Zeus_PC                 equ AppEntry                    ; Tell the emulator where to start
Zeus_SP                 equ $FF46                       ; Tell the emulator where to put the stack

; These generate some output files

                        ; Generate a SZX file, these understand 128K files
;                        output_szx AppFilename+".szx",$0000,AppEntry ; The szx file

                        ; Generate a Z80 file, these understand 128K files
;                        output_z80 AppFilename+".z80",$0000,AppEntry ; A z80 file

                        ; Generate an SNA file, but beware! Few emulators support 128K sna files.
;                        output_sna AppFilename+".sna"   ; An SNA file

                        ; And a mode 3 paged file

; I'm using a macro as I wand to repeat this with slightly different options

mBuildTZX               macro(name,options)
                          output_tzx name,AppFilename,"",AppEntry,AppLast-AppEntry,3,AppEntry,options
                          output_tzx_block name,zeuspage(0)+CodePage_0.String-$C000,CodePage_0.PageLen
                          output_tzx_block name,zeuspage(1)+CodePage_1.String-$C000,CodePage_1.PageLen
                          output_tzx_block name,zeuspage(2)+CodePage_2.String-$C000,CodePage_2.PageLen
                          output_tzx_block name,zeuspage(3)+CodePage_3.String-$C000,CodePage_3.PageLen
                          output_tzx_block name,zeuspage(4)+CodePage_4.String-$C000,CodePage_4.PageLen
                          output_tzx_block name,zeuspage(5)+CodePage_5.String-$C000,CodePage_5.PageLen
                          output_tzx_block name,zeuspage(6)+CodePage_6.String-$C000,CodePage_6.PageLen
                          output_tzx_block name,zeuspage(7)+CodePage_7.String-$C000,CodePage_7.PageLen
                          mend

                        mBuildTZX("Mode3.tzx",$00000101)
                        mBuildTZX("Mode3a.tzx",$01000101)
                        mBuildTZX("Mode3b.tzx",$02000101)
                        mBuildTZX("Mode3c.tzx",$03000101)
                        mBuildTZX("Mode3d.tzx",$04000101)
                        mBuildTZX("Mode3e.tzx",$05000101)
                        mBuildTZX("Mode3f.tzx",$06123456)

