/*
Example code for loops

Zeus supports various different kinds of loop control structure

The simplest is loop/lend

Loop/Lend is a simple way to repeat a block of lines a number of times. It does nothing
you couldn't do with a for, repeat or while loop, but it doesn't require a control
variable.

Note that if the count for a loop is zero or negative it will be ignored, i.e. the
statements inside the loop won't be used even once.

So, here are a couple of examples.
*/

; A simple loop that fills a block of memory with a pattern.
; (look at the code tab to see it)

                org $8000                               ; Somewhere convenient

                loop 32                                 ; 32 * 8 = 256 bytes
                  db $00,$01,$02,$03,$04,$05,$06,$07    ;
                lend                                    ;

; Now, two nested loops with a relative instruction inside, so the value changes.

                org $8100                               ; Somewhere convenient

Fred            loop 8                                  ; 8 outer loops
                  loop 2                                ; 2 inner loops
                    jr Fred                             ; Something that changes
                    lend                                ;
                  lend                                  ;

; Now, same again but all on one line...

                loop 8:loop 2: jr Fred:lend:lend; Neurgh!

/* I'm sure you get the idea.
Now, the next simplest loop is a for/next loop

For/Next is a way to repeat a block of instructions a number of times. It provides
an index variable, which can be useful. I'm sure you know what a for/next loop
looks like so here are some examples:
*/

                ; Let's build a table of 256 bytes from $00 to $FF

Freddy          for fred=0 to 255                       ; Notice that fred is declared here as a variable
                  db fred                               ; and the value of it is used here each time through the loop
                  next                                  ;

                ; You can have step values that are not fixed as +1, using step
                ; This runs from 255 down to 0

                for fred=255 to 0 step -1               ; Notice that fred can be reused as it's a variable
                  db fred                               ;
                  next                                  ;

                ; You can use floating point values as well, but there's a catch
                ; Zeus needs to know before the "for" that it's going to be a floating point for/next
                ; and how you do that is declare the index variable as a floating point variable.

                ; This sets up a signed 8-bit sin table, roughly speaking

angle           = 1.0                                   ; We need to declare this as a floating point value
                for angle=0.0 to 3.1415 step 0.01       ; Now we can use it here - NOTE the "0.0", "0" will be rejected.
                  db round(sin(angle)*127)              ;
                  next                                  ;

                ; In practice you'd be better off doing it like this, for 8-bit angles:

                for index=0 to 255                      ;
                  db round(sin(index*3.1415/256)*127)   ;
                  next                                  ;

/*
Anyway, that's enough about loops - except for the problem of labels inside loops.

Using external labels is fine, for example suppose we wanted to unroll a loop for speed and call a routine
a number of times. This is an easy way to do it:
*/

                loop 16                                 ; 16 copies of a call instruction
                  call Dummy                            ;
                  lend                                  ;

/*
But suppose we had the following code, which counts the number of set bits in a word.
I'm not saying this is the best way to do this, it certainly isn't, it's just an example
that's easy to understand
*/
                ; We enter with a byte in A

                ld b,8                                  ; 8 bits
                ld c,0                                  ; Result = 0

CountLp         rrca                                    ; Get the next bit
                jr nc BitNotSet                         ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet       djnz CountLp                            ; Loop

                ; We leave with the number of bits set in that byte in C

/*
Now, one way to speed this up is to "unroll the loop", or repeat the active code a number
of times instead of having a loop counter.
*/

                ; We enter with a byte in A

                ld c,0                                  ; Result = 0

                rrca                                    ; Get the next bit
                jr nc BitNotSet0                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet0      rrca                                    ; Get the next bit
                jr nc BitNotSet1                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet1      rrca                                    ; Get the next bit
                jr nc BitNotSet2                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet2      rrca                                    ; Get the next bit
                jr nc BitNotSet3                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet3      rrca                                    ; Get the next bit
                jr nc BitNotSet4                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet4      rrca                                    ; Get the next bit
                jr nc BitNotSet5                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet5      rrca                                    ; Get the next bit
                jr nc BitNotSet6                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet6      rrca                                    ; Get the next bit
                jr nc BitNotSet7                        ; If it's clear skip the increment
                inc c                                   ; This bit is set
BitNotSet7      equ *                                   ; Done

                ; We leave with the number of bits set in that byte in C

/*
That's fine, but having all those labels is tedious and error prone.

A better way would be a macro and a loop.
*/

; Define a macro to test a bit
mCountBit       macro()                                 ; We use a macro to give us a local label, essentially
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip            mend                                  ;

; Now we use it

                ; We enter with a byte in A

                ld c,0                                  ; Result = 0

                loop 8                                  ;
                  mCountBit()                           ;
                  lend                                  ;

                ; We leave with the number of bits set in that byte in C

/*

Which is less error-prone but still clumsy unless the macro is reused somewhere else.

Wouldn't it be nice if we could just do this:

                loop 8                                  ; Do this 8 times
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              equ *                                 ; After the inc
                  lend                                  ; Loop

But if you try that you'll find you get redefined label errors. These occur because you
are repeating this block of code:

                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              equ *                                 ; After the inc
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              equ *                                 ; After the inc
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              equ *                                 ; After the inc
                  ; Etc, etc

And clearly the label "Skip" is being redefined for every repeat except the first.

However, Zeus does provide a nice resolution to this problem by using something known as
an "anonymous procedure", which allows you to easily create regions of source code that
have their own local labels. You do this by enclosing that region inside a
proc/pend block.

Here's a working example of this code:
*/

                loop 8                                  ; Do this 8 times
                  proc                                  ; Start an anonymous procedure
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              equ *                                 ; After the inc
                  pend                                  ; End the procedure
                  lend                                  ; Loop

; We could tidy it up to this:

                loop 8                                  ; Do this 8 times
                  proc                                  ; Start an anonymous procedure
                  rrca                                  ; Get the next bit
                  jr nc Skip                            ; If it's clear skip the increment
                  inc c                                 ; This bit is set
Skip              pend                                  ; End the procedure
                  lend                                  ; Loop

/*
Now, just enclosing the code inside the loop in a proc/pend is enough to deal with most situations,
but there is another case that's worth discussing. Nested loops.

Loops (of any kind, for/next, loop/lend, repeat until, etc) are said to be nested if there
is a loop placed inside another loop. And of course you could also have loops inside loops
inside loops... again, the simple cases are, well, simple.
*/
                noflow                                  ; Don't warn us about executing into data

                loop 2                                  ; Produces two copies of "Hello Hello Hello Hello There!"
                  loop 4                                ; Produces "Hello Hello Hello Hello "
                    db "Hello "                         ;
                    lend                                ;
                  db "There!"                           ; Adds a "There!"
                  lend                                  ;

; But again, it's labels in the loops that can cause problems.
; This is fine, much the same as before - we can use an anonymous procedure in the inner loop.

                loop 2                                  ; Do the outer loop twice
                  loop 4                                ; And the inner loop 4 times
                    proc                                ; Start an anonymous procedure
                    rrca                                ; Get the next bit
                    jr nc Skip                          ; If it's clear skip the increment
                    inc c                               ; This bit is set
Skip                pend                                ; End the procedure
                    lend                                ; Loop
                  lend

; But sometimes we need labels in each loop, and we cannot just use one anonymous procedure
; for both loops as that way labels will get redeclared.
; Zeus will allow nested anonymous procedures (since v3.65) to handle this case, but you may
; need to understand how to address labels in different nesting levels.

; This example does not even attempt to do anything useful, just demonstrates the nesting level
; scoping operator "^^", which means "go up a nesting level"

              loop 4                                    ; Outer loop
                proc                                    ; And it has local labels
      First     nop                                     ; This is one of them
                loop 3                                  ; Inner loop
                  proc                                  ; Which also has its own local labels
      Lp          call Dummy                            ; "Lp" is local to the inner code, but "Dummy" is a global
                  djnz Lp                               ; We're just looping to the innermost "Lp"
                  jp ^^First                            ; This jumps to "First" in the outer loop
                  jp ^^Next                             ; This jumps to "Next" in the outer loop
                  pend                                  ; End the inner anonymous procedure
                  lend                                  ; End the inner loop
      Next      pend                                    ; End the outer anonymous procedure
                lend                                    ; End the outer loop

; Various support routines

Dummy           ret                                     ; Just a dummy routine

/*

One final point - you cannot as of v3.64 of Zeus use anonymous procedures inside named procedures
or macros... I might remove this limitation in future.

*/












; For backward-compatibility with other assemblers Zeus also supports "rept" .. "mend" for loops.
                rept 32                                 ; 32 * 8 = 256 bytes
                  db $00,$01,$02,$03,$04,$05,$06,$07    ;
                mend                                    ;



