/*
This is a simple demonstration file for the Zeus assembler.
If you want more sustantial files they're available at www.desdes.com


Procedures. (See also "zeus_ex_loop.asm")


From v1.4 onwards Zeus supports procedures - these are one way Zeus supports local labels.

Procedures have the following syntax:

Name    proc

        code, including local labels

        endp

Note that endp will not add a "RET" instruction, but Zeus will warn you if it
thinks you might need one.

An alternative way to end a procedure is "retp", like this:

Name    proc

        code, including local labels

        retp

In this case the retp automatically adds a return instruction.

So, an example might be:

PrintStrZ       proc            ; Start the procedure

Lp              ld a,(hl)       ; Get next
                or a            ; Zero?
                jr z End        ; Yes, done. Note! This End is local to the procedure

                call Print      ; Print this character

                inc hl          ; Loop
                jr Lp           ;

End             endp            ; endp does not add a RET instruction

The label Lp can occur outside the procedure without causing conflicts, or it could
be reused in another procedure, or macro, and so on. All labels declared inside a
procedure are local to that procedure unless Zeus is told otherwise see below.

The scoping rules for symbols are that Zeus will first try to find the symbol in the
local symbols, if there are local symbols, then it will try looking in the global symbols.

If you want Zeus to ignore local labels and only look for a global label you can use the
"::" scoping operator in front of the reference, like this: "jp ::Lp", and similarly
if you wanted to declare a global label while inside a procedure you can put "::" in front
of the label in a declaration, like this: "::AGlobalLabel equ 42"

Note - local labels are listed in the symbol table in the following form.

Local labels from procedures are listed as [procedurename]local name.
Local labels from macros are listed as [macroname][InstanceID]local name.

*/

; Example source (a lot of trouble to demonstrate a macro, BWTH)

                        zeusemulate "48K"               ; Emulate a ZX Spectrumm 48K

; Print control codes

cDel                    equ $08                         ;
cCret                   equ $0D                         ;
cCls                    equ $0C                         ;
cClr                    equ $1B                         ;
cHome                   equ $0E                         ;

; Print colour select characters

cBlack                  equ $00                         ;
cBlue                   equ $01                         ;
cRed                    equ $02                         ;
cMagenta                equ $03                         ;
cGreen                  equ $04                         ;
cCyan                   equ $05                         ;
cYellow                 equ $06                         ;
cWhite                  equ $07                         ;

                        org $8000                       ; Somewhere safe

Start                   di                              ; Make sure we're not interrupted
                        ld sp,0                         ; Set the stack to the top of memory

                        ld a,cCls                       ; Clear the screen
                        call Print                      ;

                        ld hl,-1                        ; Load a constant

                        ld a,cBlue                      ; Show the first in blue
                        call Print                      ;

                        call PrintHex_HL                ; Show the value

                        call Print_Sp                   ; " "

                        NegHL_A()                       ; Negate it

                        ld a,cRed                       ; Show the second in red
                        call Print                      ;

                        call PrintHex_HL                ; Show the value

                        call Print_Sp                   ; " "

                        call PrintStrFollow             ; Print the string following this call
                        noflow                          ; Turn the warning off
                        db cCyan," Hello there!"        ;
                        db cCret,"exciting, isn't it...",0;

                        ; We're done...

                        halt                            ; Stop all the excitement

; Print the zero-terminated string pointed to by HL

PrintStr                proc                            ;

                        push hl,af                      ;

Loop                    ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,Exit                       ;

                        call Print                      ;
                        jr Loop                         ;

Exit                    pop af,hl                       ;
                        retp                            ; (RET)

; Print the string following the call in memory

PrintStrFollow          proc                            ;

                        ex (sp),hl                      ;
                        push af                         ;

Loop                    ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z,Exit                       ;

                        call Print                      ;

                        jr Loop                         ;

Exit                    pop af                          ;
                        ex (sp),hl                      ;

                        retp                            ; (RET)


; Print the contents of HL in hex

PrintHex_HL             ld a,h                          ;
                        call PrintHex_A                 ;

                        ld a,l                          ;

                        ; Print the contents of A in hex

PrintHex_A              call PH2                        ;

PH2                     rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;

                        push af                         ;

                        and $0F                         ;

                        add a,"0"                       ;
                        cp "9"+1                        ;
                        jr c,PH3                        ;

                        add a,"A"-("9"+1)               ;

PH3                     call Print                      ;

                        pop af                          ;
                        ret                             ;

; Print a space

Print_Sp                push af                         ; Save

                        ld a,' '                        ; Print a space
                        call Print                      ;

                        pop af                          ; Done
                        ret                             ;

; A print character routine. Proportionally spaced characters

Print                   push ix,hl,de,bc,af             ; Save them

                        cp cWhite+1                     ;
                        jr c,PrintColour                ;

                        cp cCls                         ;
                        jr z,PrintCls                   ;

                        cp cCret                        ;
                        jr z,PrintCret                  ;

                        cp cHome                        ;
                        jr z,PrintHome                  ;

                        ld h,PrintWidths/256            ;
                        ld l,a                          ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        call c,PrintCret                ;

                        call PrintOutChar               ;

                        ld a,(pCurX)                    ;
                        add a,(hl)                      ;
                        ld (pCurX),a                    ;

PrintExit               pop af,bc,de,hl,ix              ;
                        ret                             ;

; Set a colour

PrintColour             cp 4                            ; Above green don't make them bright
                        jr nc,PrintCol1                 ;

                        or $40                          ; Bright

PrintCol1               ld (CurCol),a                   ;
                        jr PrintExit                    ;

; Carriage return

PrintCret               call PrintDoCret                ;
                        jr PrintExit                    ;

; Clear screen - this clears it slowly with a fade effect

PrintCls                ld e,$00                        ;
                        ld hl,$5800                     ;
                        ld bc,$02FF                     ;

pC1                     ld a,(hl)                       ;
                        and $07                         ;
                        jr z,pC2                        ;

                        set 0,e                         ;
                        dec (hl)                        ;
pC2                     cpi                             ;
                        jp pe,pC1                       ;

                        call Delay                      ;

                        bit 0,e                         ;
                        jr nz,PrintCls                  ;

                        ld hl,$4000                     ;
                        ld de,$4001                     ;
                        ld bc,$1800                     ;

                        xor a                           ;
                        ld (hl),a                       ;

                        ldir                            ;

                        ld a,(CurCol)                   ;
                        ld (hl),a                       ;

                        ld bc,$02FF                     ;

                        ldir                            ;

                        xor a                           ;
                        out ($FE),a                     ;

PrintHome               xor a                           ;
                        ld (pCurX),a                    ;
                        ld (ppCurY),a                   ;

                        ld a,cWhite                     ;
                        ld (CurCol),a                   ;

                        ld hl,$4000                     ;
                        ld (pCurY),hl                   ;

                        jr PrintExit                    ;

; A suitable delay for the fade

Delay                   push bc                         ;
                        ld bc,1000                      ;
Del1                    cpi                             ;
                        dec hl                          ;
                        jp pe,Del1                      ;
                        pop bc                          ;
                        ret                             ;

; Do the carriage return

PrintDoCret             ld a,(ppCurY)                   ;
                        inc a                           ;

PrintSetUpY             ld (ppCurY),a                   ;

                        and $1F                         ;

                        push hl                         ;

                        ld l,a                          ;
                        ld h,$00                        ;

                        add hl,hl                       ;

                        ld de,YTable                    ;
                        add hl,de                       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        ld (pCurY),hl                   ;

                        xor a                           ;
                        ld (pCurX),a                    ;

                        pop hl                          ;
                        ret                             ;

; Output the character

PrintOutChar            bit 7,l                         ; characters $00..$7F only
                        ret nz                          ;

                        push hl                         ;

                        ld h,0                          ;

                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;

                        ld de,PrintChars-256            ;
                        add hl,de                       ;

                        push hl                         ;
                        pop ix                          ;

                        ld a,(pCurX)                    ;

                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $1F                         ;

                        ld de,(pCurY)                   ;

                        or e                            ;
                        ld e,a                          ;

                        push de                         ;

                        ld a,d                          ;
                        rrca                            ;
                        rrca                            ;
                        rrca                            ;
                        and $03                         ;
                        or $58                          ;
                        ld d,a                          ;

                        ld a,(CurCol)                   ;
                        ld (de),a                       ;
                        inc de                          ;
                        ld (de),a                       ;

                        pop de                          ;

                        ld a,(pCurX)                    ;
                        and $07                         ;
                        ld c,a                          ;

                        ld b,$08                        ;

OutCharL                ld h,(ix)                       ;
                        ld l,$00                        ;

                        ld a,c                          ;
                        or a                            ;
                        jr z,OCL1                       ;

OCL2                    srl h                           ;
                        rr l                            ;
                        dec a                           ;
                        jr nz,OCL2                      ;

OCL1                    ld a,(de)                       ;
                        xor h                           ;
                        ld (de),a                       ;

                        inc e                           ;

                        ld a,(de)                       ;
                        xor l                           ;
                        ld (de),a                       ;

                        dec e                           ;

                        inc ix                          ;
                        inc d                           ;

                        djnz OutCharL                   ;

                        pop hl                          ;
                        ret                             ;

; Screen addresses for the character rows

YTable                  defw $4000,$4020,$4040,$4060    ;
                        defw $4080,$40A0,$40C0,$40E0    ;
                        defw $4800,$4820,$4840,$4860    ;
                        defw $4880,$48A0,$48C0,$48E0    ;
                        defw $5000,$5020,$5040,$5060    ;
                        defw $5080,$50A0,$50C0,$50E0    ;
                        defw 0,0,0,0,0,0,0,0            ; So running off the end isn't fatal

; Declare the macros

NegHL_A                 macro()                         ; No parameters supported (yet)

                          ld a,l                        ; Code to negate HL
                          cpl                           ;
                          ld l,a                        ;
                          ld a,h                        ;
                          cpl                           ;
                          ld h,a                        ;
                          inc hl                        ;

                          mend                          ; Macro end

InvL_A                  macro()                         ; No parameters supported (yet)

                          ld a,l                        ; Code to invert L
                          cpl                           ;
                          ld l,a                        ;

                          mend                          ; Macro end

InvH_A                  macro()                         ; No parameters supported (yet)

                          ld a,h                        ; Code to invert H
                          cpl                           ;
                          ld h,a                        ;

                          mend                          ; Macro end

; This macro includes other macros

NestedNegHL_A           macro()                         ; No parameters supported (yet)

                          InvL_A()                      ; Invert L
                          InvH_A()                      ; Invert H
                          inc hl                        ; +1

                          mend                          ; Macro end

; Character set graphics

                        align 256                       ; Put these on page boundaries

PrintChars              dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg #####---                     ;
                        dg -#-#----                     ;
                        dg #####---                     ;
                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -####---                     ;
                        dg #-#-----                     ;
                        dg -###----                     ;
                        dg --#-#---                     ;
                        dg ####----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg ##--#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #--##---                     ;
                        dg ---##---                     ;
                        dg --------                     ;

                        dg -#------                     ;
                        dg #-#-----                     ;
                        dg #-#-----                     ;
                        dg -#------                     ;
                        dg #-#-#---                     ;
                        dg #--#----                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg #-#-#---                     ;
                        dg -###----                     ;
                        dg --#-----                     ;
                        dg -###----                     ;
                        dg #-#-#---                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg #-#-#---                     ;
                        dg ##--#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -##-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg ----#---                     ;
                        dg --##----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --##----                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ---#----                     ;
                        dg --##----                     ;
                        dg -#-#----                     ;
                        dg #--#----                     ;
                        dg #####---                     ;
                        dg ---#----                     ;
                        dg ---#----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --###---                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg --------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #-###---                     ;
                        dg #-##----                     ;
                        dg #-------                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ###-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #--#----                     ;
                        dg #-#-----                     ;
                        dg ##------                     ;
                        dg #-#-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg ##-##---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ##--#---                     ;
                        dg #-#-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #--#----                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-#-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg ----#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg ##-##---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ----#---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg ##------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg ----#---                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg #####---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg ---##---                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -###----                     ;
                        dg #-#-#---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -##-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-##----                     ;
                        dg ##--#---                     ;
                        dg #---#---                     ;
                        dg ##--#---                     ;
                        dg #-##----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg ----#---                     ;
                        dg ----#---                     ;
                        dg -##-#---                     ;
                        dg #--##---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --#-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #---#---                     ;
                        dg #--##---                     ;
                        dg -##-#---                     ;
                        dg ----#---                     ;
                        dg -###----                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg #-------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg --------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ##------                     ;

                        dg #-------                     ;
                        dg #-------                     ;
                        dg #--#----                     ;
                        dg #-#-----                     ;
                        dg ###-----                     ;
                        dg #--#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg ###-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ##-#----                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg #-#-#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ####----                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg ####----                     ;
                        dg #-------                     ;
                        dg #-------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg ----#---                     ;
                        dg ----#---                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #-##----                     ;
                        dg ##------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg -####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg ----#---                     ;
                        dg ####----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --#-----                     ;
                        dg #####---                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ---##---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -####---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #---#---                     ;
                        dg #-#-#---                     ;
                        dg -#-#----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #---#---                     ;
                        dg -#-#----                     ;
                        dg --#-----                     ;
                        dg -#-#----                     ;
                        dg #---#---                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #--#----                     ;
                        dg #--#----                     ;
                        dg #--#----                     ;
                        dg ####----                     ;
                        dg ---#----                     ;
                        dg ###-----                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg #####---                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg -#------                     ;
                        dg #####---                     ;
                        dg --------                     ;

                        dg --##----                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg #-------                     ;
                        dg -#------                     ;
                        dg -#------                     ;
                        dg --##----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg #-------                     ;
                        dg #-------                     ;
                        dg --------                     ;

                        dg ##------                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ---#----                     ;
                        dg --#-----                     ;
                        dg --#-----                     ;
                        dg ##------                     ;
                        dg --------                     ;

                        dg -#-#----                     ;
                        dg --------                     ;
                        dg -###----                     ;
                        dg #---#---                     ;
                        dg #####---                     ;
                        dg #-------                     ;
                        dg -###----                     ;
                        dg --------                     ;

                        dg --------                     ;
                        dg --------                     ;
                        dg ----#---                     ;
                        dg -###----                     ;
                        dg #-------                     ;
                        dg --------                     ;
                        dg --------                     ;
                        dg --------                     ;
; Character widths

PrintWidths             defb $07,$09,$09,$09,$09,$09,$09,$03;
                        defb $05,$05,$05,$05,$03,$05,$05,$05;
                        defb $05,$03,$05,$05,$05,$05,$09,$09; " " ..
                        defb $07,$09,$07,$02,$05,$09,$07,$09;
                        defb $06,$02,$04,$06,$06,$06,$06,$02;
                        defb $04,$04,$06,$06,$03,$06,$02,$06;
                        defb $06,$06,$06,$06,$06,$06,$06,$06;
                        defb $06,$06,$02,$03,$05,$06,$05,$06;
                        defb $06,$06,$06,$06,$06,$06,$06,$06;
                        defb $06,$04,$06,$06,$06,$06,$06,$06;
                        defb $06,$06,$06,$06,$06,$06,$06,$06;
                        defb $06,$06,$06,$06,$06,$06,$06,$06;
                        defb $06,$06,$06,$06,$06,$06,$04,$06;
                        defb $06,$02,$04,$06,$04,$06,$06,$06;
                        defb $06,$06,$05,$06,$06,$06,$06,$06;
                        defb $06,$05,$06,$05,$02,$05,$06,$06;

; Variables

pCurX                   db 0                            ; A logical X position
ppCurY                  db 0                            ; A logical Y position
pCurY                   dw 0                            ; The actual address
CurCol                  db 0                            ; The cursor colour

; Create a loadable file for this, use mode 2

;                                   Filename         Name         Comment      Start  Lenght M  ExecAddr {M=Tape file mode}
                        output_tzx "procedures.tzx","procedures","A test file",$8000,*-$8000,2,Start ; Generate a tzx file

;                        export_sym "",$83FC
